==> markd-0.7.1.tar.gz: lib/src/document.dart <==
library markdown.document;

import 'ast.dart';
import 'block_parser.dart';
import 'inline_parser.dart';

/// Maintains the context needed to parse a markdown document.
class Document {
  final Map<String, Link> refLinks;
  final List<InlineSyntax> inlineSyntaxes;

  Document({inlineSyntaxes, linkResolver, imageLinkResolver})
    : refLinks = <String, Link>{},
      this.inlineSyntaxes = InlineParser.getInlineSyntaxes(
        inlineSyntaxes: inlineSyntaxes, linkResolver: linkResolver,
        imageLinkResolver: imageLinkResolver);

  Document.plain()
    : refLinks = <String, Link>{},
      this.inlineSyntaxes = InlineParser.defaultSyntaxes;

  parseRefLinks(List<String> lines) {
    // This is a hideous regex. It matches:
    // [id]: http:foo.com "some title"
    // Where there may whitespace in there, and where the title may be in
    // single quotes, double quotes, or parentheses.
    final indent = r'^[ ]{0,3}'; // Leading indentation.
    final id = r'\[([^\]]+)\]';  // Reference id in [brackets].
    final quote = r'"[^"]+"';    // Title in "double quotes".
    final apos = r"'[^']+'";     // Title in 'single quotes'.
    final paren = r"\([^)]+\)";  // Title in (parentheses).
    final pattern = new RegExp(
        '$indent$id:\\s+(\\S+)\\s*($quote|$apos|$paren|)\\s*\$');

    for (int i = 0; i < lines.length; i++) {
      final match = pattern.firstMatch(lines[i]);
      if (match != null) {
        // Parse the link.
        var id = match[1];
        var url = match[2];
        var title = match[3];

        if (title == '') {
          // No title.
          title = null;
        } else {
          // Remove "", '', or ().
          title = title.substring(1, title.length - 1);
        }

        // References are case-insensitive.
        id = id.toLowerCase();

        refLinks[id] = new Link(id, url, title);

        // Remove it from the output. We replace it with a blank line which will
        // get consumed by later processing.
        lines[i] = '';
      }
    }
  }

  /// Parse the given [lines] of markdown to a series of AST nodes.
  List<Node> parseLines(List<String> lines) {
    final parser = new BlockParser(lines, this);

    final blocks = [];
    while (!parser.isDone) {
      for (final syntax in BlockSyntax.syntaxes) {
        if (syntax.canParse(parser)) {
          final block = syntax.parse(parser);
          if (block != null) blocks.add(block);
          break;
        }
      }
    }

    return blocks;
  }

  /// Takes a string of raw text and processes all inline markdown tags,
  /// returning a list of AST nodes. For example, given ``"*this **is** a*
  /// `markdown`"``, returns:
  /// `<em>this <strong>is</strong> a</em> <code>markdown</code>`.
  List<Node> parseInline(String text) => new InlineParser(text, this).parse();
}

class Link {
  final String id;
  final String url;
  final String title;
  Link(this.id, this.url, this.title);
}

==> markd-0.8.0-dev.tar.gz: lib/src/document.dart <==
library markdown.src.document;

import 'ast.dart';
import 'block_parser.dart';
import 'inline_parser.dart';

/** Returns the URL of the given link, a [Link], a [Node],
 * or null if it is not a link.
 *
 * * [name] - the name of link, i.e., the text inside `[]`.
 * * [url] - the URL of link, i.e., the text inside `()`. Notice that
 * it is null if there is no `()` following `[]`.
 *
 * > Note: if [url] is null, you can return a [Node] instance (in additions
 * to `null` or a String instance (URL)).
 */
typedef LinkResolver(InlineParser parser, String name, String url);

/// Maintains the context needed to parse a markdown document.
class Document {
  final Map<String, Link> refLinks;
  final List<InlineSyntax> inlineSyntaxes;
  final options;

  Document({inlineSyntaxes, linkResolver, imageLinkResolver, this.options})
    : refLinks = <String, Link>{},
      this.inlineSyntaxes = InlineParser.getInlineSyntaxes(
        inlineSyntaxes: inlineSyntaxes, linkResolver: linkResolver,
        imageLinkResolver: imageLinkResolver);

  Document.plain({this.options})
    : refLinks = <String, Link>{},
      this.inlineSyntaxes = InlineParser.defaultSyntaxes;

  parseRefLinks(List<String> lines) {
    // This is a hideous regex. It matches:
    // [id]: http:foo.com "some title"
    // Where there may whitespace in there, and where the title may be in
    // single quotes, double quotes, or parentheses.
    var indent = r'^[ ]{0,3}'; // Leading indentation.
    var id = r'\[([^\]]+)\]'; // Reference id in [brackets].
    var quote = r'"[^"]+"'; // Title in "double quotes".
    var apos = r"'[^']+'"; // Title in 'single quotes'.
    var paren = r"\([^)]+\)"; // Title in (parentheses).
    var pattern =
        new RegExp('$indent$id:\\s+(\\S+)\\s*($quote|$apos|$paren|)\\s*\$');

    for (var i = 0; i < lines.length; i++) {
      var match = pattern.firstMatch(lines[i]);
      if (match != null) {
        // Parse the link.
        var id = match[1];
        var url = match[2];
        var title = match[3];

        if (title == '') {
          // No title.
          title = null;
        } else {
          // Remove "", '', or ().
          title = title.substring(1, title.length - 1);
        }

        // References are case-insensitive.
        id = id.toLowerCase();

        refLinks[id] = new Link(id, url, title);

        // Remove it from the output. We replace it with a blank line which will
        // get consumed by later processing.
        lines[i] = '';
      }
    }
  }

  /// Parse the given [lines] of markdown to a series of AST nodes.
  List<Node> parseLines(List<String> lines) {
    var parser = new BlockParser(lines, this);

    var blocks = <Node>[];
    while (!parser.isDone) {
      for (var syntax in BlockSyntax.syntaxes) {
        if (syntax.canParse(parser)) {
          var block = syntax.parse(parser);
          if (block != null) blocks.add(block);
          break;
        }
      }
    }

    return blocks;
  }

  /// Takes a string of raw text and processes all inline markdown tags,
  /// returning a list of AST nodes. For example, given ``"*this **is** a*
  /// `markdown`"``, returns:
  /// `<em>this <strong>is</strong> a</em> <code>markdown</code>`.
  List<Node> parseInline(String text) => new InlineParser(text, this).parse();
}

class Link {
  final String id;
  final String url;
  final String title;
  Link(this.id, this.url, this.title);
}
